/************************************************************************
 *   MODULE  : LOSMISC.C
 *
 *   PURPOSE : contains various routines to handle window functions
 *             in the basemap and data entry windows
 *
 *   AUTHOR  : Kevin Russo, SAIC (original, 1, 2, 3)
 *
 *   Change     Date          Reason
 *   ------     ----          ------
 *   Original   12 Aug 92     n/a
 *   1          20 Oct 92     overall program update
 *   2          03 Apr 93     update to program GUI
 *   3          24 Jun 93     update to XVT 3.0 and MUSE beta release 3
 *
 ************************************************************************/

/****************
 * header files *
 ****************/

#include "los.h"

#if 0

#if XVT_OS_ISUNIX == TRUE
#include <X11/Intrinsic.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <Mrm/MrmAppl.h>
#endif

/**************************
 * file scope definitions *
 **************************/

#if XVT_OS_ISUNIX
#define   offset         32
#else
#define   offset         0
#endif

/**********************************
 * file scope function prototypes *
 **********************************/

#if XVT_OS_ISUNIX == TRUE
#ifdef PROTO
void refresh_cmap (void);
#else
void refresh_cmap ();
#endif
#endif

/********************
 * global variables *
 ********************/

extern WIN_INFO wininfo;        /* non-muse window information          */
extern FONT_SIZE_TYPE fontsize; /* font info                            */
extern FONT /* XVT_FNTID xvt_R3FNT */  small_font;         /* times font with serifs               */
extern PREFS_TYPE sys_prefs;    /* system preferences                   */

#if 0
/************************************************************************
 *   FUNCTION     : set_scroll_bars
 *
 *   PURPOSE      : sets the scroll bar range and position for the basemap
 *                  window based upon the size of the map in reference to
 *                  the window and the zoom factor.
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void 
 *
 *   CALLED FROM  : do_file_menu in module LOSMENU1.C
 *                  do_view_menu in module LOSMENU3.C
 *                  do_basemap_win in module LOSW501.C
 *
 *   CALLS        : available_structure {MUSE LIB}
 *                  view_get_cur {MUSE LIB}
 *                  view_to_bit {MUSE LIB}
 *
 ************************************************************************/           

#ifdef PROTO
void set_scroll_bars(void)
#else
void set_scroll_bars()
#endif
{
   RCT        rct;          /* rectangle                              */
   AVAILABLE  *av=NULL;     /* muse available structure               */
   MAP_DOC    *mapdoc;      /* ptr to the basemap document            */
   BASEMAP    *basemap;     /* ptr to the basemap                     */
   GFLOAT bitx, bity;       /* x and y bits obtained from geo pos.    */

   extern MAP_TYPE mapinfo; /* non-muse map information               */

   /* get a pointer to the MUSE structure */
   available_structure(&av);
   if (av->windows->current_map == (WINDOW)NULL)
      return;

   /* get basemap pointers */
   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
      xvt_dm_post_fatal_exit("Unable to access the basemap map document");

   basemap = mapdoc->products->basemap;

   /* get the current map view */
   xvt_vobj_get_client_rect(av->windows->current_map, &rct);
   view_to_bit(av->windows->current_map, (GFLOAT)rct.right,
              (GFLOAT)rct.bottom, &bitx, &bity);

   /* set new range */
   mapinfo.org.h = 0;
   mapinfo.org.v = 0;
   mapinfo.range.h = basemap->bit_map->data_matrix.width - bitx - 1;
   mapinfo.range.v = basemap->bit_map->data_matrix.height -
                     (basemap->bit_map->data_matrix.height - bity) - 1;

   /* set the horizontal scroll bar range */
   if (mapinfo.range.h <= 0)
   {
      mapinfo.range.h = 0;
      xvt_sbar_set_range(av->windows->current_map, HSCROLL, 0, 0);
      xvt_sbar_set_pos(av->windows->current_map, HSCROLL, 0);
   }
   else
   {
      xvt_sbar_set_range(av->windows->current_map, HSCROLL, 0,
                       mapinfo.range.h);
      xvt_sbar_set_pos(av->windows->current_map, HSCROLL, mapinfo.org.h);
   }

   /* set the vertical scroll bar range */
   if (mapinfo.range.v <= 0)
   {
      mapinfo.range.v = 0;
      xvt_sbar_set_range(av->windows->current_map, VSCROLL, 0, 0);
      xvt_sbar_set_pos(av->windows->current_map, VSCROLL, 0);
   }
   else
   {
      xvt_sbar_set_range(av->windows->current_map, VSCROLL, 0,
                       mapinfo.range.v);
      xvt_sbar_set_pos(av->windows->current_map, VSCROLL, mapinfo.org.v);
   }
}

/************************************************************************
 *   FUNCTION     : draw_loc_indicators
 *
 *   PURPOSE      : draws the target and observer location on the map win.
 *                  Each is represented by a color-filled circle surrounded
 *                  by a hollow circle.
 *
 *                      RED color indicates the target location
 *                      BLUE color indicates the observer location
 *
 *   INPUTS       : obs  - flag indicating if the observer should be drawn
 *                  targ - flag indicating if the target should be drawn
 *
 *   RETURN VALUE : void 
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : available_structure {MUSE LIB}
 *                  view_get_cur {MUSE LIB}
 *                  geo_to_view {MUSE LIB}
 *                  geo_to_bit {MUSE LIB}
 *
 ************************************************************************/           

#ifdef PROTO
void draw_loc_indicators(BOOLEAN obs, BOOLEAN targ)
#else
void draw_loc_indicators(obs, targ)
BOOLEAN obs, targ;
#endif
{
   AVAILABLE *av=NULL;              /* muse available structure     */
   MAP_DOC *mapdoc;                 /* basemap document             */
   MGM *mgm;                        /* basemap geometry             */
   GFLOAT viewx, viewy, bitx, bity; /* basemap bit x,y from geo     */
   GFLOAT diff, sq1, sq2, distance; /* used to calc drawing dist    */
   DRAW_CTOOLS t;                   /* XVT brush, font, pen, etc    */
   RCT bndrct;                      /* bounding rectangles          */
   PNT pnt, endpnt;                 /* a single XVT screen pixel    */
   long scr_width;                  /* Screen width                 */
   short scaling;                   /* used to enhance locator size */

   extern MAP_TYPE mapinfo;         /* non-muse map info            */

   static COLOR col[] = { COLOR_RED, COLOR_GREEN, COLOR_BLUE,
                          COLOR_CYAN, COLOR_MAGENTA, COLOR_YELLOW,
                          COLOR_BLACK, COLOR_WHITE };

   /* check for data window existance */
   if (wininfo.lookupTable502[0] == (WINDOW)NULL)
      return;

   /* get a pointer to the MUSE structure */
   available_structure(&av);
   if (av->windows->current_map == (WINDOW)NULL)
      return;

   /* get color drawing tools for map window */
   xvt_app_get_default_ctools(&t);

   /* get basemap and map geometry pointers */
   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
      xvt_dm_post_fatal_exit("Unable to access the basemap map document");

   mgm = mapdoc->mgm;

   /* get scaling factor to account for screen size */
   scr_width = xvt_vobj_get_attr (NULL_WIN, ATTR_SCREEN_WIDTH);

   scaling = (short)((float)scr_width / (float)MIN_SCREEN_WID);
   if (scaling <= (short)ONE)
      scaling = (short)ZERO;

   /* plot the bore-sight line */
   if (targ && obs && mapinfo.boresight)
   {
      geo_to_view(mgm, av->windows->current_map, (GFLOAT)mapinfo.t.x,
                 (GFLOAT)mapinfo.t.y, &viewx, &viewy);

      endpnt.h = (short)viewx;
      endpnt.v = (short)viewy;

      geo_to_view(mgm, av->windows->current_map, (GFLOAT)mapinfo.o.x,
                 (GFLOAT)mapinfo.o.y, &viewx, &viewy);

      pnt.h = (short)viewx;
      pnt.v = (short)viewy;


      diff = fabs ((GFLOAT)endpnt.h - (GFLOAT)pnt.h);
      sq1 = diff * diff;

      diff = fabs ((GFLOAT)endpnt.v - (GFLOAT)pnt.v);
      sq2 = diff * diff;

      distance = sqrt(sq1 + sq2);

      if (distance != (GFLOAT)ZERO)
      {
         endpnt.v -= (short)((((8.0 * mapinfo.scale) + scaling) /
                     distance) * (endpnt.v - pnt.v));
         endpnt.h -= (short)((((8.0 * mapinfo.scale) + scaling) /
                     distance) * (endpnt.h - pnt.h));
      }

      t.pen.pat = PAT_SOLID;
      t.pen.color = col[sys_prefs.bor_col];
      t.pen.width = (short)mapinfo.scale * 2;
      xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */

      xvt_dwin_draw_set_pos(av->windows->current_map, pnt);

      xvt_dwin_draw_aline(av->windows->current_map, endpnt, FALSE, TRUE);
   }

   /* plot the target location */
   if (targ)
   {
      geo_to_bit(mgm, (GFLOAT)mapinfo.t.x, (GFLOAT)mapinfo.t.y,
                 &bitx, &bity);
      if ((bitx > mgm->dev_extent.left) || (bitx < mgm->dev_extent.right) ||
         (bity > mgm->dev_ymin) || (bity < mgm->dev_ymax))
      {
         geo_to_view(mgm, av->windows->current_map, (GFLOAT)mapinfo.t.x,
                    (GFLOAT)mapinfo.t.y, &viewx, &viewy);

         pnt.v = (short)viewy;
         pnt.h = (short)viewx;

         xvt_rect_set(&bndrct, pnt.h - (6*mapinfo.scale) - scaling,
                           pnt.v - (6*mapinfo.scale) - scaling,
                           pnt.h + (6*mapinfo.scale) + scaling,
                           pnt.v + (6*mapinfo.scale) + scaling);
     
         t.pen.pat = PAT_SOLID;
         t.pen.color = col[sys_prefs.tgt_col];
         t.pen.width = 1;
         t.brush.pat = PAT_SOLID;
         t.brush.color = COLOR_WHITE;
         xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */

         if (sys_prefs.tgt_shape == S_CIRCLE)
            xvt_dwin_draw_oval(av->windows->current_map, &bndrct);
         else
            xvt_dwin_draw_rect(av->windows->current_map, &bndrct);

         xvt_rect_set(&bndrct, pnt.h - (3*mapinfo.scale) - scaling,
                           pnt.v - (3*mapinfo.scale) - scaling,
                           pnt.h + (3*mapinfo.scale) + scaling,
                           pnt.v + (3*mapinfo.scale) + scaling);

         t.brush.color = col[sys_prefs.tgt_col];
         xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */

         if (sys_prefs.tgt_shape == S_CIRCLE)
            xvt_dwin_draw_oval(av->windows->current_map, &bndrct);
         else
            xvt_dwin_draw_rect(av->windows->current_map, &bndrct);
      }
   }

   /* plot the observer location */
   if (obs)
   {
      geo_to_bit(mgm, (GFLOAT)mapinfo.o.x, (GFLOAT)mapinfo.o.y,
                 &bitx, &bity);
      if ((bitx > mgm->dev_extent.left) || (bitx < mgm->dev_extent.right) ||
          (bity > mgm->dev_ymin) || (bity < mgm->dev_ymax))
      {
         geo_to_view(mgm, av->windows->current_map, (GFLOAT)mapinfo.o.x,
                    (GFLOAT)mapinfo.o.y, &viewx, &viewy);

         pnt.v = (short)viewy;
         pnt.h = (short)viewx;

         xvt_rect_set(&bndrct, pnt.h - (6*mapinfo.scale) - scaling,
                           pnt.v - (6*mapinfo.scale) - scaling,
                           pnt.h + (6*mapinfo.scale) + scaling,
                           pnt.v + (6*mapinfo.scale) + scaling);

         t.pen.pat = PAT_SOLID;
         t.pen.color = col[sys_prefs.obs_col];
         t.pen.width = 1;
         t.brush.pat = PAT_SOLID;
         t.brush.color = COLOR_WHITE;
         xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */

         if (sys_prefs.obs_shape == S_CIRCLE)
            xvt_dwin_draw_oval(av->windows->current_map, &bndrct);
         else
            xvt_dwin_draw_rect(av->windows->current_map, &bndrct);

         xvt_rect_set(&bndrct, pnt.h - (3*mapinfo.scale) - scaling,
                           pnt.v - (3*mapinfo.scale) - scaling,
                           pnt.h + (3*mapinfo.scale) + scaling,
                           pnt.v + (3*mapinfo.scale) + scaling);

         t.brush.color = col[sys_prefs.obs_col];
         xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */

         if (sys_prefs.obs_shape == S_CIRCLE)
            xvt_dwin_draw_oval(av->windows->current_map, &bndrct);
         else
            xvt_dwin_draw_rect(av->windows->current_map, &bndrct);
      }
   }

   xvt_app_get_default_ctools(&t);
   xvt_dwin_set_draw_ctools(av->windows->current_map, &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(av->windows->current..., font_id); */
}

/************************************************************************
 *   FUNCTION     : flip_radio
 *
 *   PURPOSE      : turns one of two radio buttons on, turns the other off
 *
 *   INPUTS       : win        - the data entry window identifier
 *                  ctrl_1_win - id to the radio button to turn on
 *                  ctrl_2_win - id to the radio button to turn off
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_control in module LOSW502.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void flip_radio(WINDOW win, WINDOW ctrl_1_win, WINDOW ctrl_2_win)
#else
void flip_radio(win, ctrl_1_win, ctrl_2_win)
WINDOW win, ctrl_1_win, ctrl_2_win;
#endif
{
   WINDOW ctls[2];                      /* control windows              */

   ctls[0] = ctrl_1_win;
   ctls[1] = ctrl_2_win;
   xvt_ctl_check_radio_button (win, ctls, 2);
}

/************************************************************************
 *   FUNCTION     : reset_text_edits
 *
 *   PURPOSE      : fills text edits with previous setting values
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_control in module LOSW502.C
 *                  do_data_win in module LOSW502.C
 *                  do_reset_menu in module LOSMENU2.C
 *                  do_file_menu in module LOSMENU1.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void reset_text_edits()
#else
void reset_text_edits()
#endif
{
    POINT_DD dd;                        /* decimal degrees              */
    POINT_UNITS user_unit;              /* new units                    */
    GFLOAT feet;                        /* new measurement in feet      */

    char buf[21];                       /* text edit string value       */
    long loop;                          /* loop counter                 */
    short elev;                         /* elevation at the obs/tgt loc */
    
    extern SETTINGS_TYPE settings;      /* user entered params          */

    /* suspend text edit events */
    for (loop = DATA_WIN_START; loop <= DATA_WIN_RANGE; loop++)
       xvt_tx_suspend ((TXEDIT)wininfo.lookupTable502[loop]);

    /* set the range text edit */
    if (sys_prefs.dist_units == METERS)
       sprintf(buf, "%ld", (long)settings.range);
    else
    {
       feet = settings.range * FEET_PER_METER;
       sprintf(buf, "%ld", (long)feet);
    }
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_RANGE]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_RANGE], 0, buf);

    /* set the target height text edit */
    if (sys_prefs.dist_units == METERS)
    {
       if (sys_prefs.hgt_rel == AMSL)
       {
          dd.lat = settings.target.loc.y;
          dd.lon = settings.target.loc.x;

          elev = obsv_elev (dd.lat, dd.lon);

          if (elev < (short)ZERO)
             sprintf(buf, "UNKNOWN IN AMSL");
          else
             sprintf(buf, "%.4lf",
                      (GFLOAT)((GFLOAT)elev + settings.target.height));
       }
       else
          sprintf(buf, "%.4lf", settings.target.height);
    }
    else
    {
       feet = settings.target.height * FEET_PER_METER;

       if (sys_prefs.hgt_rel == AMSL)
       {
          dd.lat = settings.target.loc.y;
          dd.lon = settings.target.loc.x;

          elev = obsv_elev (dd.lat, dd.lon);

          if (elev < (short)ZERO)
             sprintf(buf, "UNKNOWN IN AMSL");
          else
          {
             feet += ((GFLOAT)elev * FEET_PER_METER);
             sprintf(buf, "%.4lf", feet);
          }
       }
       else
          sprintf(buf, "%.4lf", feet);
    }
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_HGT]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_HGT], 0, buf);

    /* set the target width text edit */
    if (sys_prefs.dist_units == METERS)
       sprintf(buf, "%.4lf", settings.target.width);
    else
    {
       feet = settings.target.width * FEET_PER_METER;
       sprintf(buf, "%.4lf", feet);
    }
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_WID]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_WID], 0, buf);

    /* set the target latitude and longitude text edits */
    if (sys_prefs.loc_units == DD_STYLE)
    {
       if (settings.target.loc.x > (GFLOAT)ZERO)
          sprintf(buf, "%.4lf E", settings.target.loc.x);
       else if (settings.target.loc.x < (GFLOAT)ZERO)
          sprintf(buf, "%.4lf W", (GFLOAT)-1*settings.target.loc.x);
       else
          sprintf(buf, "000.0000");

       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_LON]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_LON], 0, buf);

       if (settings.target.loc.y > (GFLOAT)ZERO)
          sprintf(buf, "%.4lf N", settings.target.loc.y);
       else if (settings.target.loc.y < (GFLOAT)ZERO)
          sprintf(buf, "%.4lf S", (GFLOAT)-1*settings.target.loc.y);
       else
          sprintf(buf, "00.0000");
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_LAT]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_LAT], 0, buf);
    }
    else
    {
       dd.lat = settings.target.loc.y;
       dd.lon = settings.target.loc.x;

       geo_conv_to (DMS, &dd, &user_unit);

       if (user_unit.dms.lon.d < (short)ZERO)
          sprintf(buf, "%03d %02d %02.2lf W", (short)(-1*user_unit.dms.lon.d),
                   user_unit.dms.lon.m, user_unit.dms.lon.s);
       else
          sprintf(buf, "%03d %02d %02.2lf E", user_unit.dms.lon.d,
                   user_unit.dms.lon.m, user_unit.dms.lon.s);
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_LON]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_LON], 0, buf);


       if (user_unit.dms.lat.d < (short)ZERO)
          sprintf(buf, "%02d %02d %02.2lf S", (short)(-1*user_unit.dms.lat.d),
                   user_unit.dms.lat.m, user_unit.dms.lat.s);
       else
          sprintf(buf, "%02d %02d %02.2lf N", user_unit.dms.lat.d,
                   user_unit.dms.lat.m, user_unit.dms.lat.s);
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_T_LAT]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_T_LAT], 0, buf);
    }

    /* set the observer height text edit */
    if (sys_prefs.dist_units == METERS)
    {
       if (sys_prefs.hgt_rel == AMSL)
       {
          dd.lat = settings.observer.loc.y;
          dd.lon = settings.observer.loc.x;

          elev = obsv_elev (dd.lat, dd.lon);

          if (elev < (short)ZERO)
             sprintf(buf, "UNKNOWN IN AMSL");
          else
             sprintf(buf, "%.4lf",
                      (GFLOAT)((GFLOAT)elev + settings.observer.height));
       }
       else
          sprintf(buf, "%.4lf", settings.observer.height);
    }
    else
    {
       feet = settings.observer.height * FEET_PER_METER;

       if (sys_prefs.hgt_rel == AMSL)
       {
          dd.lat = settings.observer.loc.y;
          dd.lon = settings.observer.loc.x;

          elev = obsv_elev (dd.lat, dd.lon);

          if (elev < (short)ZERO)
             sprintf(buf, "UNKNOWN IN AMSL");
          else
          {
             feet += ((GFLOAT)elev * FEET_PER_METER);
             sprintf(buf, "%.4lf", feet);
          }
       }
       else
          sprintf(buf, "%.4lf", feet);
    }
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_O_HGT]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_O_HGT], 0, buf);

    /* set the observer latitude and longitude text edit */
    if (sys_prefs.loc_units == DD_STYLE)
    {
       if (settings.observer.loc.x > (GFLOAT)ZERO)
          sprintf(buf, "%.4lf E", settings.observer.loc.x);
       else if (settings.observer.loc.x < (GFLOAT)ZERO)
          sprintf(buf, "%.4lf W", (GFLOAT)-1*settings.observer.loc.x);
       else
          sprintf(buf, "000.0000");
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_O_LON]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_O_LON], 0, buf);

       if (settings.observer.loc.y > (GFLOAT)ZERO)
          sprintf(buf, "%.4lf N", settings.observer.loc.y);
       else if (settings.observer.loc.y < (GFLOAT)ZERO)
          sprintf(buf, "%.4lf S", (GFLOAT)-1*settings.observer.loc.y);
       else
          sprintf(buf, "00.0000");
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_O_LAT]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_O_LAT], 0, buf);
    }
    else
    {
       dd.lat = settings.observer.loc.y;
       dd.lon = settings.observer.loc.x;

       geo_conv_to (DMS, &dd, &user_unit);

       if (user_unit.dms.lon.d < (short)ZERO)
          sprintf(buf, "%03d %02d %02.2lf W", (short)(-1*user_unit.dms.lon.d),
                   user_unit.dms.lon.m, user_unit.dms.lon.s);
       else
          sprintf(buf, "%03d %02d %02.2lf E", user_unit.dms.lon.d,
                   user_unit.dms.lon.m, user_unit.dms.lon.s);
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_O_LON]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_O_LON], 0, buf);


       if (user_unit.dms.lat.d < (short)ZERO)
          sprintf(buf, "%02d %02d %02.2lf S", (short)(-1*user_unit.dms.lat.d),
                   user_unit.dms.lat.m, user_unit.dms.lat.s);
       else
          sprintf(buf, "%02d %02d %02.2lf N", user_unit.dms.lat.d,
                   user_unit.dms.lat.m, user_unit.dms.lat.s);
       xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_O_LAT]);
       xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_O_LAT], 0, buf);
    }

    /* set the azimuth end text edit */
    sprintf(buf, "%.4lf", settings.end);
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_END]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_END], 0, buf);

    /* set the azimuth start text edit */
    sprintf(buf, "%.4lf", settings.start);
    xvt_tx_clear(wininfo.lookupTable502[DATA_WIN_START]);
    xvt_tx_add_par(wininfo.lookupTable502[DATA_WIN_START], 0, buf);

    /* resume text edit events */
    for (loop = DATA_WIN_START; loop <= DATA_WIN_RANGE; loop++)
       xvt_tx_resume ((TXEDIT)wininfo.lookupTable502[loop]);
}

/************************************************************************
 *   FUNCTION     : status_update
 *
 *   PURPOSE      : displays a message within the basemap status window
 *
 *   INPUTS       : message - char string to display
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void status_update(char *message)
#else
void status_update(message)
char *message;
#endif
{
   DRAW_CTOOLS t;                             /* XVT drawing tools    */
   RCT rct;                                   /* rectangle            */

   extern WIN_INFO wininfo;                   /* non-muse window info */
   extern MAP_TYPE mapinfo;                   /* non-muse map info    */

   if (wininfo.lookupTable504[0] == (WINDOW)NULL)
      return;

   /* set the drawing tools */
   xvt_app_get_default_ctools(&t);
   t.font = small_font;
   t.fore_color = COLOR_YELLOW;
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
   t.back_color = COLOR_GRAY;
#elif XVT_OS == XVT_OS_MAC
   t.back_color = COLOR_DKGRAY;
#else
   t.back_color = COLOR_DKGRAY;
#endif
   t.opaque_text = TRUE;
   xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */

   /* write new message to screen */
   mapinfo.status[0] = (char)NULL;

   if (message[0] != (char)NULL)
      sprintf(mapinfo.status, "%s", message);

   xvt_vobj_get_client_rect (wininfo.lookupTable504[0], &rct);
   xvt_rect_set (&rct, rct.left, rct.bottom - 10 - fontsize.size,
                   rct.right, rct.bottom);
   xvt_dwin_invalidate_rect (wininfo.lookupTable504[0], &rct);
   xvt_dwin_update (wininfo.lookupTable504[0]);
}

/************************************************************************
 *   FUNCTION     : ChartXY
 *
 *   PURPOSE      : Displays the lat and lon based on cursor pos within
 *                  the basemap
 *
 *   INPUTS       : type  - the event type
 *                  where - the screen position of the pointer
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_mouse_* in module LOSW501.C
 *
 *   CALLS        : available_structure {MUSE LIB},
 *                  view_to_bit {MUSE LIB},
 *                  bit_to_geo {MUSE LIB}
 *
 ************************************************************************/

#ifdef PROTO
coord_type ChartXY(PNT where)
#else
coord_type ChartXY(where)
PNT where;
#endif
{
    AVAILABLE *av=NULL;                            /* MUSE main struct  */
    DRAW_CTOOLS t;                                 /* drawing tools     */
    RCT rct;                                       /* MBR rectangles    */
    PNT mousepos;                                  /* cursor position   */
    GFLOAT viewx, viewy, bitx, bity, latdd, londd; /* basemap info      */
    MAP_DOC *mapdoc;                               /* ptr to map doc    */
    MGM *mgm;                                      /* map geometry      */
    POINT_DD dd;                                   /* decimal degrees   */
    POINT_UNITS user_unit;                         /* new units         */
    coord_type coord;                              /* geo pos of cursor */

    int pixlen;                                    /* strlen in pixels  */
    char lat_text[80], lon_text[80];               /* new lat/lon text  */

    extern MAP_TYPE mapinfo;                       /* NON-MUSE map info */

    /* initialize the geo pos of mouse */
    coord.x = ERRCOORD;
    coord.y = ERRCOORD;

    if (wininfo.lookupTable502[0] == (WINDOW)NULL)
       return coord;

    /* get the MUSE main structure */
    available_structure(&av);
    if(av->windows->current_map == (WINDOW)NULL)
       return coord;
    xvt_vobj_get_client_rect(av->windows->current_map, &rct);

    mousepos.h = where.h;
    mousepos.v = where.v;

    /* is mouse within the map window? */
    if (xvt_rect_has_point(&rct, mousepos))
    {
       mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
       if (mapdoc == (MAP_DOC *)NULL)
          xvt_dm_post_fatal_exit("Unable to access the basemap map document");

       mgm = mapdoc->mgm;

       /* get the current map view geometry */
       viewx = where.h;
       viewy = where.v;
       view_to_bit(av->windows->current_map, viewx, viewy, &bitx, &bity);

       /* determine if the mouse location is within the map extents */
       if ((bitx >= mgm->dev_extent.left) && (bitx <= mgm->dev_extent.right)
          && (bity >= mgm->dev_ymin) && (bity <= mgm->dev_ymax))
       {
          bit_to_geo(mgm, bitx, bity, &londd, &latdd);

          coord.y = mapinfo.cursor_pos.y = latdd;
          coord.x = mapinfo.cursor_pos.x = londd;

          if (sys_prefs.loc_units == DD_STYLE)
          {
             if (latdd > 0)
                sprintf(lat_text, "%07.4lf N  ",(GFLOAT)latdd);
             else if (latdd < 0)
                sprintf(lat_text, "%07.4lf S  ",(GFLOAT)-1*latdd);
             else
                sprintf(lat_text, "00.0000    ");

             if (londd > 0)
                sprintf(lon_text, "%08.4lf E  ",(GFLOAT)londd);
             else if (londd < 0)
                sprintf(lon_text, "%08.4lf W  ",(GFLOAT)-1*londd);
             else
                sprintf(lon_text, "000.0000    ");
          }
          else
          {
             dd.lat = latdd;
             dd.lon = londd;

             geo_conv_to (DMS, &dd, &user_unit);

             if (user_unit.dms.lon.d < (short)ZERO)
                sprintf(lon_text, "%03d %02d %02.2lf W     ",
                         (short)(-1*user_unit.dms.lon.d),
                         user_unit.dms.lon.m, user_unit.dms.lon.s);
             else
                sprintf(lon_text, "%03d %02d %02.2lf E     ",
                         user_unit.dms.lon.d, user_unit.dms.lon.m,
                         user_unit.dms.lon.s);

             if (user_unit.dms.lat.d < (short)ZERO)
                sprintf(lat_text, "%02d %02d %02.2lf S     ",
                         (short)(-1*user_unit.dms.lat.d),
                         user_unit.dms.lat.m, user_unit.dms.lat.s);
             else
                sprintf(lat_text, "%02d %02d %02.2lf N     ",
                         user_unit.dms.lat.d, user_unit.dms.lat.m,
                         user_unit.dms.lat.s);
          }

          /* get the clipping rect and window size */
          xvt_vobj_get_client_rect(wininfo.lookupTable504[0], &rct);

          /* set the drawing tools */
          xvt_app_get_default_ctools(&t);
          t.font = small_font;
          t.fore_color  = COLOR_WHITE;
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
          t.back_color  = COLOR_GRAY;
#elif XVT_OS == XVT_OS_MAC
          t.back_color  = COLOR_DKGRAY;
#else
          t.back_color  = COLOR_DKGRAY;
#endif
          t.opaque_text = TRUE;
          xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */

          /* set the new text */
          pixlen = xvt_dwin_get_text_width (wininfo.lookupTable504[0],
                   "Cursor Longitude:     ", -1);
          xvt_rect_set(&rct, rct.left+5, rct.top+5, rct.right-5, rct.bottom-5);
          xvt_dwin_draw_text(wininfo.lookupTable504[0],
                rct.left+pixlen+10, rct.top+fontsize.size, lat_text, -1);
          xvt_dwin_draw_text(wininfo.lookupTable504[0],
                rct.left+pixlen+10, rct.top+(2*fontsize.size), lon_text, -1);
       }
    }

    return coord;
}

#endif

#endif

/************************************************************************
 *   FUNCTION     : sphere_to_xyz
 *
 *   PURPOSE      : converts spherical coords to cartesian coords
 *
 *   INPUTS       : geo - the geographic position
 *                  xyz - pointer to the returned cartesian coords
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : gt_circle in module LOSMISC.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void sphere_to_xyz (coord_type geo, xyz_type *xyz)
#else
void sphere_to_xyz (geo, xyz)
coord_type geo;
xyz_type *xyz;
#endif
{
   double phi_rad, lambda_rad, q;        /* used in calculations        */
   double x=0, y=0, z=0;                 /* used in calculations        */

   phi_rad = geo.y/DEG_PER_RAD;
   lambda_rad = geo.x/DEG_PER_RAD;

   q = EARTH_RADIUS * cos(phi_rad);

   x = q * cos (lambda_rad);
   y = q * sin (lambda_rad);
   z = EARTH_RADIUS * sin (phi_rad);

   if ( fabs(x) <= 1.0e-3)      x = 0;
   if ( fabs(y) <= 1.0e-3)      y = 0;
   if ( fabs(z) <= 1.0e-3)      z = 0;

   xyz->x = x;
   xyz->y = y;
   xyz->z = z;
}

/************************************************************************
 *   FUNCTION     : gt_circle
 *
 *   PURPOSE      : returns the great circle distance between two
 *                  geographic positions
 *
 *   INPUTS       : pt1 - the first geographic position
 *                  pt2 - the second geographic position
 *
 *   RETURN VALUE : float - the great circle distance
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : sphere_to_xyz in module LOSMISC.C
 *
 ************************************************************************/

#ifdef PROTO
GFLOAT gt_circle ( coord_type pt1, coord_type pt2 )
#else
GFLOAT gt_circle ( pt1, pt2 )
coord_type pt1, pt2;
#endif
{
   xyz_type xyz1, xyz2;                      /* used in calculations    */
   double dd, xx, cosang, ang;               /* used in calculations    */

   /* convert from spherical to cartesian */
   sphere_to_xyz (pt1, &xyz1);
   sphere_to_xyz (pt2, &xyz2);

   /* calculate great circle diatnce */
   dd = ( (xyz2.x - xyz1.x)*(xyz2.x - xyz1.x) +
          (xyz2.y - xyz1.y)*(xyz2.y - xyz1.y) +
          (xyz2.z - xyz1.z)*(xyz2.z - xyz1.z) );

   xx = 2 * EARTH_RADIUS * EARTH_RADIUS;

   cosang = (xx - dd)/xx;

   if (cosang > 1)
      cosang=1;

   if (cosang < -1)
      cosang=-1;

   ang = acos(cosang);

   return (GFLOAT)(ang * EARTH_RADIUS);
}

#if 0
/************************************************************************
 *   FUNCTION     : map_redraw
 *
 *   PURPOSE      : redraws the map with the same scale and translation
 *                  as in the previous map
 *
 *   INPUTS       : raster     - pointer to the basemap raster bitmap
 *                  map_window - the window on which to draw the map
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : popen_struct {MUSE LIB},
 *                  pcell_array {MUSE LIB},
 *                  pscale {MUSE LIB},
 *                  ptranslate {MUSE LIB},
 *                  ppost_struct {MUSE LIB},
 *                  pclose_struct {MUSE LIB},
 *                  draw_loc_indicators in module LOSMISC.C
 *
 ************************************************************************/

#ifdef PROTO
void map_redraw(RASTER *raster, WINDOW map_window)
#else
void map_redraw(raster, map_window)
RASTER *raster;
WINDOW map_window;
#endif
{
#if XVT_OS == XVT_OS_MAC
   MAP_DOC          *map_doc;           /* basemap map document         */
#endif

   extern  MAP_TYPE mapinfo;

   /* Phigs variables */
   Ppat_rep         colrs;              /* color array                  */
   Prect            corners;            /* MBR of the image             */
   Pfloat           priority = 1.0;     /* drawing priority             */
   Pvec             scale, trans;       /* scale and translation vals   */
   Pint             error_id = 0;       /* phigs error code             */
   Pmatrix          m;                  /* plotting matrix              */

   if (raster->data_matrix.data_type != COLOR_INDIRECT)
   {
     /*      xvt_dm_post_error("The raster should have contained COLOR_INDIRECT.");*/
      return;
   }

#if XVT_OS == XVT_OS_MAC
   map_doc = (MAP_DOC *)xvt_vobj_get_data(map_window);

   if (map_doc == (MAP_DOC *)NULL)
      xvt_dm_post_fatal_exit("MAP_DOC problem in map_redraw");

   xvt_dwin_clear (map_window, COLOR_WHITE);
#endif

   /* stretch the MBR to the image */
   corners.p.x = corners.p.y = 0.0;
   corners.q.x = raster->data_matrix.width;
   corners.q.y = raster->data_matrix.height;

   /* lock down the color array in the bitmap */
   colrs.dims.size_x = raster->data_matrix.width;
   colrs.dims.size_y = raster->data_matrix.height;
   colrs.colr_array = (BYTE pHMEM) xvt_gmem_lock(raster->data_matrix.ghandle);

   if (colrs.colr_array==pHMEM_NULL)
      xvt_dm_post_fatal_exit("Out of memory in map_redraw()");

   colrs.pix_per_byte = BITS_PER_BYTE / raster->data_matrix.bits_per_pixel;

   /* open the structure */
   popen_struct(map_window);

#if XVT_OS == XVT_OS_MAC
   pput_pal(map_window, map_doc->palette);
#endif

   /* set the scale and translation factors */
   pcell_array (&corners, colrs);
   scale.delta_x = scale.delta_y = 1.0;
   pscale(scale, error_id, m);
   scale.delta_x = scale.delta_y = (float)mapinfo.scale;
   pscale(scale, error_id, m);
   trans.delta_x = (Pfloat)mapinfo.org.h;
   trans.delta_y = (Pfloat)mapinfo.org.v;
   ptranslate(trans, error_id, m);

   /* close the image structure */
   pclose_struct();

   /* post the image to the screen */
   ppost_struct(map_window, map_window, priority);

   xvt_gmem_unlock(raster->data_matrix.ghandle);

#if XVT_OS == XVT_OS_WIN
   xvt_dwin_invalidate_rect (wininfo.lookupTable501[0], NULL);

   xvt_app_process_pending_events();
#elif XVT_OS_ISUNIX == TRUE
   refresh_cmap();
#endif

   return;
}

/************************************************************************
 *   FUNCTION     : update_range
 *
 *   PURPOSE      : indicates the new range to target on the basemap
 *                  status window
 *
 *   INPUTS       : length - length to target in meters
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void update_range (GFLOAT length)
#else
void update_range (length)
GFLOAT length;
#endif
{
   DRAW_CTOOLS t;                        /* drawing tools               */
   GFLOAT feet;                          /* new measure int feet        */
   RCT rct;                              /* window rectangle size       */
   char dist_text[80];                   /* new range text              */
   int len;

   extern MAP_TYPE mapinfo;              /* NON-MUSE map info           */

   if (wininfo.lookupTable502[0] == (WINDOW)NULL)
      return;

   /* get the drawing window rectangle */
   xvt_vobj_get_client_rect(wininfo.lookupTable504[0], &rct);
   xvt_rect_set(&rct, rct.left+5, rct.top+5, rct.right-5, rct.bottom-5);

   /* set the drawing tools */
   xvt_app_get_default_ctools(&t);
   t.font = small_font;
   t.fore_color = COLOR_WHITE;
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
   t.back_color = COLOR_GRAY;
#elif XVT_OS == XVT_OS_MAC
   t.back_color = COLOR_DKGRAY;
#else
   t.back_color = COLOR_DKGRAY;
#endif
   t.opaque_text = TRUE;
   xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */

   /* set the target distance */
   sprintf(dist_text, "%.4lf", length);
   mapinfo.dist = (GFLOAT)atof(dist_text);

   /* display the new target distance message */
   if (sys_prefs.dist_units == METERS)
      sprintf(dist_text, "%11ld m          ", (long)mapinfo.dist);
   else
   {
      feet = mapinfo.dist * FEET_PER_METER;
      sprintf(dist_text, "%11ld ft          ", (long)feet);
   }

   len = xvt_dwin_get_text_width(wininfo.lookupTable504[0], "Range To Target:     ", -1);
   xvt_dwin_draw_text(wininfo.lookupTable504[0],
         rct.left+len+10, rct.top+(4*fontsize.size), dist_text, -1);
}

/************************************************************************
 *   FUNCTION     : update_elevation
 *
 *   PURPOSE      : indicates the elevation at the cursor positon
 *
 *   INPUTS       : lat - latitude of the cursor
 *                  lon - longitude of the cursor
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : do_mouse_* in module LOSW501.C
 *
 *   CALLS        : obsv_elev in module ELEVATIO.C
 *
 ************************************************************************/

#ifdef PROTO
void update_elevation (GFLOAT lat, GFLOAT lon)
#else
void update_elevation (lat, lon)
GFLOAT lat, lon;
#endif
{
   DRAW_CTOOLS t, saveTools;            /* drawing tools                */
   GFLOAT feet;                         /* new measure in feet          */
   RCT         rct;                     /* window rectangle size        */
   int len;

   short elevation;

   static char elev_text[255] = { "\0" };

   extern RASTER elev_raster;

   /* check if we can provide this info */
   if (elev_raster.data_matrix.ghandle == (GHANDLE)NULL)
      return;

   if (wininfo.lookupTable504[0] == (WINDOW)NULL)
      return;

   /* save old drawing tools and drawing window*/
   xvt_dwin_get_draw_ctools(wininfo.lookupTable504[0], &saveTools);
		/* xvt_R3FNT you may also need: font_id = xvt_dwin_get_font(wininfo.lookupTable5...); */

   /* set the drawing window */
   xvt_vobj_get_client_rect(wininfo.lookupTable504[0], &rct);
   xvt_rect_set(&rct, rct.left+5, rct.top+5, rct.right-5, rct.bottom-5);

   /* set the drawing tools */
   xvt_app_get_default_ctools(&t);
   t.font = small_font;
   t.fore_color = COLOR_WHITE;
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
   t.back_color = COLOR_GRAY;
#elif XVT_OS == XVT_OS_MAC
   t.back_color = COLOR_DKGRAY;
#else
   t.back_color = COLOR_DKGRAY;
#endif
   t.opaque_text = TRUE;
   xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */

   /* get the elevation at the cursor pos */
   elevation = (short)obsv_elev (lat, lon);

   /* display the new target distance message */
   if (elevation >= (short)ZERO)
   {
      if (sys_prefs.dist_units == METERS)
         sprintf(elev_text, "%5d m          ",elevation);
      else
      {
         feet = (GFLOAT)elevation * FEET_PER_METER;
         sprintf(elev_text, "%5d ft          ", (short)feet);
      }
   }
   else
      sprintf(elev_text, "Unknown          ");

   len = xvt_dwin_get_text_width(wininfo.lookupTable504[0], "Elevation (DTED):     ", -1);
   xvt_dwin_draw_text(wininfo.lookupTable504[0],
         rct.left+len+10, rct.top+(3*fontsize.size), elev_text, -1);

   /* reset the drawing tools */
   xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &saveTools);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */
}

/************************************************************************
 *   FUNCTION     : update_vertical_acc
 *
 *   PURPOSE      : indicates the vertical accuracy on the basemap
 *                  status window
 *
 *   INPUTS       : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : update in module LOSW504.C,
 *                  do_accuracy_dlg in module LOSD103.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void update_vertical_acc(void)
#else
void update_vertical_acc()
#endif
{
   DRAW_CTOOLS t;                        /* drawing tools               */
   RCT rct;                              /* window rectangle size       */
   char acc_text[80];                    /* vertical accuracy text      */

   extern SETTINGS_TYPE settings;        /* used-supplied settings      */

   if (wininfo.lookupTable502[0] == (WINDOW)NULL)
      return;

   /* get the drawing window rectangle */
   xvt_vobj_get_client_rect(wininfo.lookupTable504[0], &rct);
   xvt_rect_set(&rct, rct.left+5, rct.top+5, rct.right-5, rct.bottom-5);

   /* set the drawing tools */
   xvt_app_get_default_ctools(&t);
   t.font = small_font;
   t.fore_color = COLOR_WHITE;
#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN2 || XVT_OS == XVT_OS_WIN3
   t.back_color = COLOR_GRAY;
#elif XVT_OS == XVT_OS_MAC
   t.back_color = COLOR_DKGRAY;
#else
   t.back_color = COLOR_DKGRAY;
#endif
   t.opaque_text = TRUE;
   xvt_dwin_set_draw_ctools(wininfo.lookupTable504[0], &t);
		/* xvt_R3FNT you may also need: xvt_dwin_set_font(wininfo.lookupTable5..., font_id); */

   if (settings.vert_acc != (long)ONE)
      sprintf(acc_text, "DTED Vertical Accuracy:     %ld meters    ",
               (long)settings.vert_acc);
   else
      sprintf(acc_text, "DTED Vertical Accuracy:     %ld meter     ",
               (long)settings.vert_acc);

   /* display the new target distance message */

   xvt_dwin_draw_text(wininfo.lookupTable504[0],
         rct.left+10, rct.top+(5*fontsize.size), acc_text, -1);
}

#endif

/************************************************************************
 *   FUNCTION     : BoxWithin 
 *
 *   PURPOSE      : determines if one geographic box lies within another
 *
 *   INPUTS       : box1 - first rectangle
 *                  box2 - second rectangle
 *
 *   RETURN VALUE : BOOLEAN - TRUE if Box1 is Within Box2
 *                          - FALSE otherwise
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
BOOLEAN BoxWithin ( EXTENT box1, EXTENT box2 )
#else
BOOLEAN BoxWithin (box1, box2)
EXTENT box1, box2;
#endif
{
   /* Test if directly inside */

   if ((box1.left >= box2.left) && (box1.left <= box2.right) &&
      (box1.bottom >= box2.bottom) && (box1.bottom <= box2.top))
      return TRUE;

   if ((box1.right >= box2.left) && (box1.right <= box2.right) &&
      (box1.bottom >= box2.bottom) && (box1.bottom <= box2.top))
      return TRUE;

   if ((box1.right >= box2.left) && (box1.right <= box2.right) &&
      (box1.top >= box2.bottom) && (box1.top <= box2.top))
      return TRUE;

   if ((box1.left >= box2.left) && (box1.left <= box2.right) &&
      (box1.top >= box2.bottom) && (box1.top <= box2.top))
      return TRUE;

   /* Test for overlaps */

   if ((box1.left <= box2.right) && (box1.right >= box2.left) &&
      (box1.bottom <= box2.top) && (box1.top >= box2.bottom))
      return TRUE;

   if ((box1.left >= box2.right) && (box1.right <= box2.left) &&
      (box1.bottom <= box2.top) && (box1.top >= box2.bottom))
      return TRUE;

   /* Box1 is not within Box2 */

   return FALSE;
}

#if 0

/************************************************************************
 *   FUNCTION     : DrawMap 
 *
 *   PURPOSE      : dispatches calls to the map drawing functions
 *
 *   INPUTS       : msg      - message to the user
 *                  load_map - flag indicating if map was just loaded
 *                             (has no palette)
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void DrawMap (char *msg, BOOLEAN load_map)
#else
void DrawMap (msg, load_map)
char *msg;
BOOLEAN load_map;
#endif
{
   AVAILABLE *av=NULL;              /* MUSE main structure                */
   BASEMAP   *basemap;              /* Pointer to the basemap             */
   MAP_DOC   *mapdoc;               /* Pointer to the basemap map doc     */
   
#if XVT_OS == XVT_OS_MAC
   VIEW             *view;          /* holds the basemap view which is    */
                                    /* used to reset the color palette    */
#endif                                    

   NOREF (load_map);

   /* get a pointer to the main MUSE structure */
   available_structure (&av);
   if (av->windows->current_map == (WINDOW)NULL)
      xvt_dm_post_fatal_exit("No map is open");
      
   /* get basemap and map document pointers */
   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
      xvt_dm_post_fatal_exit("Unable to access the basemap map document");
 
   basemap = mapdoc->products->basemap;

   if (msg[0] != (char)NULL)
      status_update (msg);

   xvt_scr_set_busy_cursor();

#if XVT_OS == XVT_OS_MAC
   xvt_scr_set_busy_cursor();
   view_get_cur(av->windows->current_map, &view);
   view->status = (int)ZERO;
#endif

/*   if (load_map)
   {
      raster_draw(basemap->bit_map, av->windows->current_map);

#if XVT_OS == XVT_OS_WIN
      xvt_dwin_invalidate_rect (wininfo.lookupTable501[0], NULL);
      xvt_app_process_pending_events();
#elif XVT_OS_ISUNIX == TRUE
      refresh_cmap();
#endif
   }
   else */
      map_redraw(basemap->bit_map, av->windows->current_map);
}

/************************************************************************
 *   FUNCTION     : check_file_name 
 *
 *   PURPOSE      : determines if a given filename is valid (uses the DOS
 *                  file naming convention [8 char. basename + 3 char. ext])
 *
 *   INPUTS       : filename - the name of the given file
 *                  filetype - the 3 char. extension which filename should
 *                             contain to be valid
 *
 *   RETURN VALUE : BOOLEAN - TRUE if valid filename,
 *                          - FALSE otherwise
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
BOOLEAN check_file_name (char *name, char *filetype)
#else
BOOLEAN check_file_name (name, filetype)
char *name, *filetype;
#endif
{
   BOOLEAN legal;

   int     position, loop;
   char    ch, extension[5], *pos, filename[15];

   if (strlen(name) <= 12)
      strcpy (filename, name);
   else
      strncpy (filename, name, 12);

   pos = strchr (filename, '.');
   if (pos == (char *)NULL)
   {
       xvt_scr_beep();
       /*       xvt_dm_post_error("No extension in file name");*/
       return FALSE;
   }

   position = (int)(pos - filename);
 
   if (position > 8)
   {
      xvt_scr_beep();
      /*      xvt_dm_post_error("file basename too long (8 chars max)");*/
      return FALSE;
   }

   position++;
   if ((strlen(filename) - position) > 3)
   {
      xvt_scr_beep();
      /*      xvt_dm_post_error("file extension too long (3 chars max)");*/
      return FALSE;
   }

   pos++;
   strcpy (extension, pos);

   if (xvt_str_match (filetype, extension, FALSE) == FALSE)
   {
      xvt_scr_beep();
      /*      xvt_dm_post_error("file extension must be : '%s'", filetype);*/
      return FALSE;
   }
               
   for (loop = 0, legal = TRUE; loop < strlen(filename); loop++)
   {
      ch = filename[loop];
 
      if ( (!isalnum(ch))  && (ch != '_') && (ch != '^') &&
           (ch != '$') && (ch != '~') && (ch != '!') && (ch != '#') &&
           (ch != '%') && (ch != '&') && (ch != '-') && (ch != '{') &&
           (ch != '}') && (ch != '(') && (ch != ')') && (ch != '@') &&
           (ch != '.') && (ch != 69)  && (ch != 96))
      {
         legal = FALSE;
         break;
      }
   }

   if (!legal)
   {
      xvt_scr_beep();
      /*      xvt_dm_post_error("Illegal characters in file name");    */
      return FALSE;
   } 

   return TRUE;
}

#endif

/************************************************************************
 *   FUNCTION     : rightjust
 *
 *   PURPOSE      : right justifies a string
 *
 *   INPUTS       : character string to justify
 *
 *   OUTPUTS      : character string right-justified
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void rightjust( char *str )
#else
void rightjust(str)
char *str;
#endif
{
   short len;
 
   len = strlen(str) - 1;

   while ((len > 0) && (str[len] == ' ')) len--;

   if (len < (strlen(str)-1))
      str[len+1] = '\0';
}

/************************************************************************
 *   FUNCTION     : leftjust
 *
 *   PURPOSE      : left justifies a string
 *
 *   INPUTS       : character string to justify
 *
 *   OUTPUTS      : character string left-justified
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : multiple places in the LOS code
 *
 *   CALLS        : none
 *
 ************************************************************************/

#ifdef PROTO
void leftjust( char *str )
#else
void leftjust(str)
char *str;
#endif
{
   short i=0, j, k=0;
 
   while (str[i] == ' ') i++;

   for (j = i; j < strlen(str); j++)
   {
      str[k] = str[j];
      k += 1;
   }

   str[k] = '\0';

   for (i = 0; i < strlen(str); i++)
      if (str[i] == '\n')
         str[i] = '\0';
}

#if 0

/************************************************************************
 *   FUNCTION     : gen_color_table
 *
 *   PURPOSE      : builds the new palette
 *
 *   INPUTS       : none
 *
 *   OUTPUTS      : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : update in module LOSUPDAT.C
 *
 *   CALLS        : none
 *
 ************************************************************************/

#if XVT_OS == XVT_OS_DOS || XVT_OS == XVT_OS_WIN
#ifdef PROTO
void gen_color_table(void)
#else
void gen_color_table()
#endif
{
   AVAILABLE *av=NULL;
   MAP_DOC *mapdoc;

   PALETTEENTRY pe[216];

   long ci;

   HANDLE       hMem;
   HDC          hdc;
   LPLOGPALETTE LogPal;

   extern HPALETTE hPalette;

   extern BININFO bins[216];

   available_structure (&av);
   if (av == (AVAILABLE *)NULL)
      return;

   if (av->windows->current_map == (WINDOW)NULL)
      return;

   mapdoc = (MAP_DOC *)xvt_vobj_get_data(av->windows->current_map);
   if (mapdoc == (MAP_DOC *)NULL)
      return;

   if (hPalette)
   {
      for (ci = 0; ci < mapdoc->palette->num_colors; ci++)
      {
         pe[ci].peFlags = PC_RESERVED;
         pe[ci].peRed   = (unsigned char)(bins[ci].red / 257);
         pe[ci].peGreen = (unsigned char)(bins[ci].green / 257);
         pe[ci].peBlue  = (unsigned char)(bins[ci].blue / 257);
      }

      AnimatePalette (hPalette, (UINT)PAL_OFFSET, (UINT)216, (const PALETTEENTRY far *)pe);
   }
   else
   {
      ci = 0;

      hMem = GlobalAlloc ( GMEM_MOVEABLE, sizeof(LOGPALETTE) +
                         (long)256 * sizeof(PALETTEENTRY) );

      if (hMem == (HANDLE)NULL)
         xvt_dm_post_fatal_exit("Out of memory in gen_color_table()");

      LogPal = (LPLOGPALETTE)GlobalLock(hMem);

      if (LogPal == (LPLOGPALETTE)NULL)
         xvt_dm_post_fatal_exit("Unable to lock down palette memory in gen_color_table()");

      LogPal->palVersion = 0x0300;
      LogPal->palNumEntries = (WORD)256;

      hdc = (HDC)xvt_vobj_get_attr(av->windows->current_map, ATTR_NATIVE_GRAPHIC_CONTEXT);

      GetSystemPaletteEntries (hdc, 0, 256, (PALETTEENTRY far *)LogPal->palPalEntry);

      for (ci = 0; ci < (long)mapdoc->palette->num_colors; ci++)
      {
         LogPal->palPalEntry[ci+PAL_OFFSET].peFlags = PC_RESERVED;
         LogPal->palPalEntry[ci+PAL_OFFSET].peRed   = (unsigned char)(mapdoc->palette->rgb[ci].r / 257);
         LogPal->palPalEntry[ci+PAL_OFFSET].peGreen = (unsigned char)(mapdoc->palette->rgb[ci].g / 257);
         LogPal->palPalEntry[ci+PAL_OFFSET].peBlue  = (unsigned char)(mapdoc->palette->rgb[ci].b / 257);
      }

      if ((hPalette = CreatePalette((const LOGPALETTE far *)LogPal)) == (HPALETTE)NULL)
         xvt_dm_post_fatal_exit("Unable to create a system palette"); 

      if (GlobalUnlock(hMem) != (BOOL)ZERO)
         xvt_dm_post_fatal_exit("Unable to unlock palette mem in gen_color_table()");

      if (GlobalFree(hMem) != (HANDLE)NULL)
         xvt_dm_post_fatal_exit("Unable to free palette mem in gen_color_table()");
   }
}
#endif

/************************************************************************
 *   FUNCTION     : refresh_cmap
 *
 *   PURPOSE      : restores the appropriate contrast within Motif
 *
 *   INPUTS       : none
 *
 *   OUTPUTS      : none
 *
 *   RETURN VALUE : void
 *
 *   CALLED FROM  : DrawMap in module LOSMISC.C,
 *                  map_redraw in module LOSMISC.C
 *
 *   CALLS        : none
 ************************************************************************/

#if XVT_OS_ISUNIX == TRUE
#ifdef PROTO
void refresh_cmap(void)
#else
void refresh_cmap()
#endif
{
   Display *dpy;

   Widget  shell;

   Window  win;

   int     scn;

   Visual  *vis;

   XVisualInfo  visinfo;
#if XVT_OS == XVT_OS_HPUX
   XVisualInfo *vislist, vtemp;
   int         vismatched;
   int         i;
#endif

   short loop;

   extern short contrast;

   extern BININFO bins[216];

   extern XColor colrs[256];
 
   extern Colormap cmap;
 
   shell = (Widget)xvt_vobj_get_attr(wininfo.lookupTable501[0], ATTR_X_WIDGET);
   dpy   = XtDisplay (shell);
   scn   = DefaultScreen(dpy);
   win   = (Window)xvt_vobj_get_attr(wininfo.lookupTable501[0], ATTR_NATIVE_WINDOW);

   if (cmap == (Colormap)NULL)
   {
#if XVT_OS == XVT_OS_HPUX
      vtemp.depth = 8;
      vtemp.screen = scn;
      vtemp.class=PseudoColor;
      vislist = XGetVisualInfo(dpy,VisualScreenMask | VisualClassMask 
                                | VisualDepthMask, &vtemp, &vismatched);
      if(vismatched != 0)
      {
        cmap = DefaultColormap(dpy, scn);

        for (i = 0; i < 255; i++)
           colrs[i].pixel = i;

        XQueryColors(dpy, cmap, colrs, 255);

        if(vismatched > 1)
          cmap = XCreateColormap(dpy, XRootWindow(dpy,
	          vislist[1].screen), vislist[1].visual, AllocAll);
        else
	  cmap = XCreateColormap(dpy, XRootWindow(dpy,
	           vislist[0].screen), vislist[0].visual, AllocAll);

	 XStoreColors(dpy, cmap, colrs, 255);

         XSetWindowColormap(dpy, win, cmap);
      }
#else
      if (XMatchVisualInfo(dpy, scn, 8, PseudoColor, &visinfo))
      {
         vis = visinfo.visual;

         cmap = DefaultColormap(dpy, scn);

         for (loop = 0; loop < 256; loop++)
            colrs[loop].pixel = loop;

         XQueryColors(dpy, cmap, colrs, 256);

         cmap = XCreateColormap(dpy, win, vis, AllocAll);

         XStoreColors(dpy, cmap, colrs, 256);

         XSetWindowColormap(dpy, win, cmap);

         XInstallColormap(dpy, cmap);
      }
#endif
      else
         xvt_dm_post_fatal_exit("XMatchVisualInfo failed");
   }
 
   if (contrast < 0)
   {
      for (loop = 0; loop < 216; loop++)
      {
         colrs[loop].red   = (unsigned short)maxof(0,
                             (bins[loop].red + (contrast * 257)));
         colrs[loop].green = (unsigned short)maxof(0,
                             (bins[loop].green + (contrast * 257)));
         colrs[loop].blue  = (unsigned short)maxof(0,
                             (bins[loop].blue + (contrast * 257)));
         colrs[loop].pixel = loop + PAL_OFFSET;
         colrs[loop].flags = DoRed | DoGreen | DoBlue;
      }
   }
   else if (contrast > 0)
   {
      for (loop = 0; loop < 216; loop++)
      {
         colrs[loop].red   = (unsigned short)minof(65535,
                             (bins[loop].red + (contrast * 257)));
         colrs[loop].green = (unsigned short)minof(65535,
                             (bins[loop].green + (contrast * 257)));
         colrs[loop].blue  = (unsigned short)minof(65535,
                             (bins[loop].blue + (contrast * 257)));
         colrs[loop].pixel = loop + PAL_OFFSET;
         colrs[loop].flags = DoRed | DoGreen | DoBlue;
      }
   }
   else
   {
      for (loop = 0; loop < 216; loop++)
      {
         colrs[loop].red   = (unsigned short)bins[loop].red;
         colrs[loop].green = (unsigned short)bins[loop].green;
         colrs[loop].blue  = (unsigned short)bins[loop].blue;
         colrs[loop].pixel = loop + PAL_OFFSET;
         colrs[loop].flags = DoRed | DoGreen | DoBlue;
      }
   }

   loop = 216;

   colrs[loop].red   = (unsigned short)65535;
   colrs[loop].green = (unsigned short)0;
   colrs[loop].blue  = (unsigned short)0;
   colrs[loop].pixel = 32;
   colrs[loop].flags = DoRed | DoGreen | DoBlue;

   colrs[loop + 1].red   = (unsigned short)0;
   colrs[loop + 1].green = (unsigned short)65535;
   colrs[loop + 1].blue  = (unsigned short)0;
   colrs[loop + 1].pixel = 33;
   colrs[loop + 1].flags = DoRed | DoGreen | DoBlue;

   colrs[loop + 2].red   = (unsigned short)0;
   colrs[loop + 2].green = (unsigned short)0;
   colrs[loop + 2].blue  = (unsigned short)65535;
   colrs[loop + 2].pixel = 34;
   colrs[loop + 2].flags = DoRed | DoGreen | DoBlue;

   colrs[loop + 3].red   = (unsigned short)65535;
   colrs[loop + 3].green = (unsigned short)65535;
   colrs[loop + 3].blue  = (unsigned short)0;
   colrs[loop + 3].pixel = 35;
   colrs[loop + 3].flags = DoRed | DoGreen | DoBlue;

   colrs[loop + 4].red   = (unsigned short)0;
   colrs[loop + 4].green = (unsigned short)65535;
   colrs[loop + 4].blue  = (unsigned short)65535;
   colrs[loop + 4].pixel = 36;
   colrs[loop + 4].flags = DoRed | DoGreen | DoBlue;

   colrs[loop + 5].red   = (unsigned short)65535;
   colrs[loop + 5].green = (unsigned short)0;
   colrs[loop + 5].blue  = (unsigned short)65535;
   colrs[loop + 5].pixel = 37;
   colrs[loop = 5].flags = DoRed | DoGreen | DoBlue;

   XStoreColors(dpy, cmap, colrs, 222);
   XSetWindowColormap(dpy, win, cmap);
   XInstallColormap(dpy, cmap);
}
#endif

#endif
