/*
 * Copyright 1997 Vectaport Inc.
 * Copyright 1994, 1995 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifndef mapcomps_h
#define mapcomps_h

#include <OverlayUnidraw/ovcomps.h>
#include <FrameUnidraw/framecomps.h>
#include <Map/mapfeature.h>

class MapCoverage;
class MapDatabase;
class MapFeature;
class MapFeatureClass;
class MapLibrary;

class MapFeatureComp : public OverlayComp {
public:
    MapFeatureComp(MapFeature* =nil, OverlayComp* parent=nil);
    MapFeatureComp(istream& in, MapFeature::MapFeatureType type, 
		   OverlayComp* parent=nil);
    virtual ~MapFeatureComp();

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    void feature(MapFeature*);
    MapFeature* feature();
    void graphic(Graphic*);

  // companion point comp containing coordinate for text comp
  void pointcomp(MapFeatureComp* mfc) { _pointcomp = mfc; }
  MapFeatureComp* pointcomp() { return _pointcomp; }

  virtual void update(Observable*);

protected:
    virtual GraphicComp* GetGraphicComp(Graphic*);

protected:
    MapFeature* _feature;
  MapFeatureComp* _pointcomp;

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_feat_params;
};


class MapFClassComp : public OverlaysComp, public IndexedGsMixin {
public:
    MapFClassComp(MapFeatureClass* fcl = nil );
    MapFClassComp(istream&, OverlayComp* parent = nil);
    virtual ~MapFClassComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    MapFeatureClass* GetFeatureClass() { return _mapfc; }
    void SetFeatureClass(MapFeatureClass* fcl) { _mapfc = fcl; }

    virtual void SetPathName(const char*);
    virtual const char* GetPathName();
    unsigned char* bitmap(float scale, int& ncols, int& nrows,
			  float& xorigin, float& yorigin, int& utmzone);

    virtual void GrowIndexedGS(Graphic* gr) { grow_indexed_gs(gr); }
    virtual void ResetIndexedGS() { reset_indexed_gs(); }
    virtual Graphic* GetIndexedGS(int i) { return get_indexed_gs(i); }

protected:
    MapFeatureClass* _mapfc;
    char* _pathname;

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_fcl_params;
};

class MapCovComp : public OverlaysComp {
public:
    MapCovComp(MapCoverage* cov = nil);
    MapCovComp(istream&);
    virtual ~MapCovComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    MapCoverage* GetCoverage() { return _mapcov; }
    MapFClassComp* FindFClassComp(MapFeatureClass*);

protected:
    virtual MapFClassComp* CreateFClassComp(MapFeatureClass*);
    MapCoverage* _mapcov;

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_cov_params;
};

class MapLibComp : public OverlaysComp {
public:
    MapLibComp(MapLibrary* lib = nil);
    MapLibComp(istream&);
    virtual ~MapLibComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    MapLibrary* GetLibrary() { return _maplib; }
    MapCovComp* FindCovComp(MapCoverage*);

protected:
    virtual MapCovComp* CreateCovComp(MapCoverage*);
    MapLibrary* _maplib;

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_lib_params;
};

class MapDbComp : public OverlaysComp {
public:
    MapDbComp(MapDatabase* db = nil);
    MapDbComp(istream&);
    virtual ~MapDbComp();

    virtual void Interpret(Command*);
    virtual void Uninterpret(Command*);

    virtual Component* Copy();
    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    MapDatabase* GetDatabase() { return _mapdb; }
    void SetDatabase(MapDatabase* db) { _mapdb = db; }
    MapLibComp* FindLibComp(MapLibrary*);
protected:
    virtual MapLibComp* CreateLibComp(MapLibrary*);
    MapDatabase* _mapdb;

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_db_params;
};

class MapIdrawComp : public FrameIdrawComp {
public:
    MapIdrawComp(const char* pathname = nil, OverlayComp* parent = nil);
    MapIdrawComp(boolean add_bg, const char* pathname = nil, OverlayComp* parent = nil);
    MapIdrawComp(istream&, const char* pathname = nil, OverlayComp* parent = nil);

    virtual ClassId GetClassId();
    virtual boolean IsA(ClassId);

    Component* Copy();

    const char* GetProjectionName();
    void SetProjectionName(char* p);

protected:
    ParamList* GetParamList();
    void GrowParamList(ParamList*);
    static ParamList* _map_idraw_params;
    static int _projection_symid;
};

#endif
